package timeTable.dataStructures;

import java.util.Vector;
import timeTable.transportNetwork.*;
import java.io.*;

public class TimeTable implements Serializable
{
    static private final long serialVersionUID = 42L;

    /************************************************/
    
    private Vector<Student> students;
    
    /************************************************/
    
    private Vector<TimeInterval> timeIntervals;
    
    /************************************************/
    
    private Vector<TimeIntervalGroup> timeIntervalGroups;
    
    /************************************************/
    
    private Vector<Availability> availabilities;
    
    /************************************************/
    
    private String fileName;
    
    /************************************************/
    
    public TimeTable()
    {	
	students = new Vector<Student>();
	timeIntervals = new  Vector<TimeInterval>();
	timeIntervalGroups = new Vector<TimeIntervalGroup>();
	availabilities = new Vector<Availability> ();
    }
    
    /************************************************/

    public void setFileName(String fileName)
    {
	this.fileName = fileName;
    }
    
    /************************************************/

    public void addStudent(String name)
    {
	add(new Student(students.size(), name));
    }
    
    /************************************************/
    
    private void add(Student student)
    {
	students.add(student);
    }
    
    /************************************************/
    
    public void addTimeInterval(String name)
    {
	add(new TimeInterval(timeIntervals.size(), name));
    }
    
    /************************************************/
    
    private void add(TimeInterval timeInterval)
    {
	timeIntervals.add(timeInterval);
    }
    
    /************************************************/
    
    private void addTimeIntervalGroup(String name)
    {
	add(new TimeIntervalGroup(name));
    }
    
    /************************************************/
    
    private void add(TimeIntervalGroup timeIntervalGroup)
    {
	timeIntervalGroups.add(timeIntervalGroup);
    }
    
    /************************************************/
    
    public void addAvailability(int studentIndex, int timeIntervalIndex)
    {
	addAvailability(students.get(studentIndex),
			timeIntervals.get(timeIntervalIndex));
    }
    
    /************************************************/
    
    public void removeAvailability(int studentIndex, int timeIntervalIndex)
    {
	Availability availability = null;
	int i = 0;
	while (i < availabilities.size() && availability == null)
	    if(availabilities.get(i).getStudentIndex() != studentIndex
	       ||  availabilities.get(i).getTimeIntervalIndex() != timeIntervalIndex)
		    i++;
	    else
		    availability = availabilities.get(i);
	if (availability != null)
	    removeAvailability(availability);
    }
    
    /************************************************/
    
    public void removeAvailability(Availability availability)
    {
	availabilities.remove(availability);
	availability.getStudent().removeAvailability(availability);
	availability.getTimeInterval().removeAvailability(availability);
    }
    
    /************************************************/
    
    private void addAvailability(Student student, TimeInterval timeInterval)
    {
	add(new Availability(student, timeInterval));
    }
    
    /************************************************/
    
    private void addAvailabilities(Student student, 
				  TimeIntervalGroup timeIntervalGroup)
    {
	int nb = timeIntervalGroup.nbTimeIntervals();
	for(int i = 0 ; i < nb ; i++)
	    addAvailability(student, 
			    timeIntervalGroup.getTimeInterval(i));
    }
    
    /************************************************/
    
    private void add(Availability availability)
    {
	availabilities.add(availability);
    }
    
    /************************************************/
    
    public void changeNumberOfStudents(int numberOfStudents)
    {
	while (numberOfStudents > getStudents().size())
	    {
		addStudent("" + (getStudents().size() + 1) + "-th student");
	    }
	if (numberOfStudents < getStudents().size())
	    {
		int aIndex = 0;
		Availability a;
		while(aIndex < availabilities.size())
		    { 
			a = availabilities.get(aIndex);
			if (numberOfStudents < a.getStudent().getIndex() + 1)
			    {
				availabilities.remove(a);
				a.getStudent().removeAvailability(a);
				a.getTimeInterval().removeAvailability(a);
			    }
			else
			    aIndex++;
		    }
	    }
	while (numberOfStudents < getStudents().size())
	    {
		getStudents().removeElementAt(getStudents().size() - 1);
	    }
    }
    
    /************************************************/
    
    public void changeNumberOfTimeIntervals(int numberOfTimeIntervals)
    {
	while (numberOfTimeIntervals > getTimeIntervals().size())
	    {
		addTimeInterval("" + (getTimeIntervals().size() + 1) + "-th time interval");
	    }
	if (numberOfTimeIntervals < getTimeIntervals().size())
	    {
		int aIndex = 0;
		Availability a;
		while(aIndex < availabilities.size())
		    { 
			a = availabilities.get(aIndex);
			if (numberOfTimeIntervals < getTimeIntervals().size() + 1)
			    {
				availabilities.remove(a);
				a.getStudent().removeAvailability(a);
				a.getTimeInterval().removeAvailability(a);
			    }
			else
			    aIndex++;
		    }
	    }
	while (numberOfTimeIntervals < getTimeIntervals().size())
	    {
		getTimeIntervals().removeElementAt(getTimeIntervals().size() - 1);
	    }
    }
    
    /************************************************/
    
    public Vector<Student> getStudents()
    {
	return students;
    }
    
    /************************************************/
    
    public Vector<TimeInterval> getTimeIntervals()
    {
	return timeIntervals;
    }
    
    /************************************************/
    
    public void computeSolution()
    {
	TransportNetwork transportNetwork = new TransportNetwork();
	Vector<Vertex> studentVertices = new Vector<Vertex>();
	int vertexIndex = 0;
	for(int i = 0 ; i < students.size() ; i++, vertexIndex++)
	    studentVertices.add(new Vertex(transportNetwork, vertexIndex));
	Vector<Vertex> timeIntervalVertices = new Vector<Vertex>();
	for(int i = 0 ; i < timeIntervals.size() ; i++, vertexIndex++)
	    timeIntervalVertices.add(new Vertex(transportNetwork, 
						vertexIndex));
	Vertex source = new Vertex(transportNetwork, vertexIndex);
	vertexIndex++;
	transportNetwork.setSource(source);
	Vertex sink = new Vertex(transportNetwork, vertexIndex);
	vertexIndex++;
	transportNetwork.setSink(sink);
	transportNetwork.setNbVertices(vertexIndex);
	Vector<Arc> arcs = new Vector<Arc>();
	int arcIndex = 0;
	for(int i = 0 ; i < availabilities.size() ; i++, arcIndex++)
	    {
		int studentIndex = availabilities.get(i).getStudentIndex();
		Vertex studentVertex = studentVertices.get(studentIndex);
		int timeIntervalIndex = 
		    availabilities.get(i).getTimeIntervalIndex();
		Vertex timeIntervalVertex = 
		    timeIntervalVertices.get(timeIntervalIndex);
		arcs.add(new Arc(studentVertex, timeIntervalVertex, 0, 1));
	    }
	for(int i = 0 ; i < studentVertices.size() ; i++, arcIndex++)
	    arcs.add(new Arc(source, studentVertices.get(i), 0, 1));
	for(int i = 0 ; i < timeIntervalVertices.size() ; i++, arcIndex++)
	    arcs.add(new Arc(timeIntervalVertices.get(i), sink, 0, 1));
	transportNetwork.setNbArcs(arcIndex);
	transportNetwork.runFlow();
	for(int i = 0 ; i < availabilities.size() ; i++)
	    if(arcs.get(i).getFlowValue() != 0)
		availabilities.get(i).chose();
    }

    
    /************************************************/
    
    public String toString()
    {
	String res = "Students : " + students;
	res += "\nTime intervals : " + timeIntervals;
	res += "\n" + availabilities;
	return res;
    }
  
    /************************************************/
    
    public boolean writeObject(String fileName)
    {
	this.fileName = fileName;
	boolean returnValue = true;
	ObjectOutput out = null;
	try
	    {
		out = new ObjectOutputStream
		    (new FileOutputStream(fileName));
		out.writeObject(this);
	    }
	catch(IOException e)
	    {
		System.out.println(e);
		returnValue = false;
	    }
	try
	    {
		out.close();
	    }
	catch(IOException e)
	    {
		System.out.println(e);
		returnValue = false;
	    }
	return true;
    }
    
    /************************************************/
    
    public boolean writeObject()
    {
	return writeObject(fileName);
    }
    
    /************************************************/
    
    public static TimeTable readObject(String fileName) 
	throws ClassNotFoundException, IOException
    {
        ObjectInputStream objstream = new ObjectInputStream
	    (new FileInputStream(fileName));
        TimeTable tt = (TimeTable)(objstream.readObject());
        objstream.close();
        return tt;
    }
  
    /************************************************/
    
    public static void main(String[] args)
    {
	TimeTable tt = new TimeTable();
	tt.addStudent("Terminator");
	tt.addStudent("Alien");
	tt.addStudent("Predator");
	tt.addStudent("Freddy");
	tt.addTimeInterval("monday");
	tt.addTimeInterval("tuesday");
	tt.addTimeInterval("wednesday");
	tt.addTimeInterval("thursday");
	tt.addAvailability(0, 0);
	tt.addAvailability(1, 0);
	tt.addAvailability(2, 0);
	tt.addAvailability(3, 0);
	tt.addAvailability(1, 1);
	tt.addAvailability(2, 1);
	tt.addAvailability(3, 1);
	tt.addAvailability(2, 3);
	tt.addAvailability(3, 2);
	tt.computeSolution();
	System.out.println(tt);	
    }
}
