package timeTable.transportNetwork;

import java.util.Vector;

/**
   Interface pour les flots, plus pr&eacute;cis&eacute;ment pour 
   les sommets.<BR>
   Les sommets qui vont nous permettre d'ex&eacute;cuter les 
   algorithmes de flots doivent juste nous permettre de parcourir le 
   cha&icirc;nage et accepter une marque. 
 */

public interface FlowableVertex
{
    /**
      Retourne le sommet source du r&eacute;seau de transport. 
     */

    public FlowableVertex getSource();
    
    /**
       Retourne le sommet puit du r&eacute;seau de transport.
    */

    public FlowableVertex getSink();
    
    /**
       Renvoie, consign&eacute;es dans un <CODE>Vector</CODE>, 
       toutes les ar&ecirc;tes entrants dans le sommet courant.
       Attention&nbsp;: il faut que tous les &eacute;l&eacute;ments 
       contenus dans ce <CODE>Vector</CODE> soient de type 
       <CODE>FlowableArc</CODE>, sinon &ccedil;a sera &agrave; 
       vous de rattraper la <CODE>ClassCastException</CODE>&nbsp;!
     */

    public Vector getInArcs();
    
    /**
       Renvoie, consign&eacute;es dans un <CODE>Vector</CODE>, 
       toutes les ar&ecirc;tes sortant du sommet courant. 
       Attention&nbsp;: il faut que tous les 
       &eacute;l&eacute;ments contenus dans ce <CODE>Vector</CODE> 
       soient de type <CODE>FlowableArc</CODE>, 
       sinon &ccedil;a sera &agrave; vous de 
       rattraper la <CODE>ClassCastException</CODE>&nbsp;!
     */
    public Vector getOutArcs();
    
    /**
      Pour placer une marque dans le sommet courant. 
    */
    public void acceptMark(FlowMark myMark);
    
    /**
       Pour consulter la marque du sommet courant.
     */
    public FlowMark getMark();

    /**
       Ajoute ce sommet dans la coupe.
     */

    public void addToTheCut();

    /**
       Retourne <CODE>true</CODE> si ce sommet est dans la coupe.
     */

    public boolean isInTheCut();

}
