package timeTable.transportNetwork;

import java.util.Vector;

/**
   Exemple d'impl&eacute;mentation de FlowableVertex, repr&eacute;sente 
   un sommet dans le r&eacute;seau de transport. Le co&ucirc;t de cet 
   impl&eacute;mentation est le passage de l'instance de 
   TransportNetwork en param&egrave;tre dans le constructeur.
 */

public class Vertex implements FlowableVertex
{

    /**
       Indice du sommet.
     */

    private int index;

    /**
       R&eacute;seau de transport auquel ce sommet est associ&eacute;.
     */

    TransportNetwork myTransportNetwork;
    
    /**
       Ensemble des ar&ecirc;tes sortantes.
     */
    
    private Vector<Arc> outArcs;
    
    /**
       Ensemble des ar&ecirc;tes entrantes.
     */
    
    private Vector<Arc> inArcs;

    /**
       Marque. 
     */
    
    private FlowMark myMark;
    
    /**
       Vrai si le sommet marqu&eacute; est 
       dans la coupe.
     */
    
    private boolean isInTheCut = false;

    /*----------------------------------------------------------------------------*/
    
    /**
       Cr&eacute;ation d'un sommet...
     */

    public Vertex(TransportNetwork myTransportNetwork, int indexOfVertex)
    {
	this.myTransportNetwork = myTransportNetwork;
	this.index = indexOfVertex;
	outArcs = new Vector<Arc>();
	inArcs = new Vector<Arc>();
    }
    
    /*----------------------------------------------------------------------------*/
    
    /**
       Retourne l'<CODE>index</CODE> du sommet courant.
     */
    
    public int getIndex()
    {
	return index;
    }

    /*----------------------------------------------------------------------------*/
    
    /**
       Ajout d'une ar&ecirc;te conforme.  
     */
    
    public void addOutArc(Arc myArc)
    {
	outArcs.add(myArc);
    }

    /*----------------------------------------------------------------------------*/
    
    /**
       Ajout d'une ar&ecirc;te inconforme. 
     */

    public void addInArc(Arc myArc)
    {
	inArcs.add(myArc);
    }

    /*----------------------------------------------------------------------------*/
    
    /**
       Retourne le sommet source.
     */

    public FlowableVertex getSource()
    {
	return myTransportNetwork.getSource();
    }
    
    /*----------------------------------------------------------------------------*/
    
    /**
       Retourne le sommet puits.
     */

    public FlowableVertex getSink()
    {
	return myTransportNetwork.getSink();
    }

    /*----------------------------------------------------------------------------*/
    
    /**
       Retourne la liste des pr&eacute;d&eacute;cesseurs.
     */

    public Vector getInArcs()
    {
	return inArcs;
    }

    /*----------------------------------------------------------------------------*/
    
    /**
       Retourne la liste des successeurs.
     */

    public Vector getOutArcs()
    {
	return outArcs;
    }

    /*----------------------------------------------------------------------------*/

    /**
       Accepte la marque myMark.
     */

    public void acceptMark(FlowMark myMark)
    {
	this.myMark = myMark;
    }
    
    /*----------------------------------------------------------------------------*/

    /**
       Retourne la marque si elle existe.
     */

    public FlowMark getMark()
    {
	return this.myMark;
    }

    /*----------------------------------------------------------------------------------------*/

    /**
       Ajoute ce sommet dans la coupe.
     */

    public void addToTheCut()
    {
	isInTheCut = true;
    }    

    /*----------------------------------------------------------------------------------------*/

    /**
       Retourne <CODE>true</CODE> si ce sommet est dans la coupe.
     */

    public boolean isInTheCut()
    {
	return isInTheCut;
    }    

    /*----------------------------------------------------------------------------*/

    /**
       Renvoie cette ar&ecirc;te sous format cha&icirc;ne de caract&egrave;res
     */

    public String toString()
    {
	String res = "{ index = " + index;
	res += " arcs = "+ outArcs;	    
	if (myMark!= null)
	    {
		res += " mark = " + myMark;
	    }
	else
	    {
		res += null;
	    }
	res += " };";
	return res;
    }   

}
